/*
 * This file is part of SessionJ.
 *
 * Copyright (C) Raymond Hu, Imperial College London.
 *
 */

package sj.ast.typenodes;

import polyglot.ast.TypeNode_c;
import polyglot.util.CodeWriter;
import polyglot.util.Position;
import polyglot.visit.PrettyPrinter;

import sj.types.typeobjects.sesstypes.SJType;

import static sj.SJConstants.SJ_STRING_SEPARATOR;

public abstract class SJTypeNode_c extends TypeNode_c implements SJTypeNode
{
	private SJTypeNode child;

	public SJTypeNode_c(Position pos)
	{
		super(pos);
	}

	public SJTypeNode child()
	{
		return child;
	}

	public SJTypeNode child(SJTypeNode child)
	{
		this.child = child;

		return this;
	}

	public SJType type()
	{
		SJTypeNode child = child();

		SJType sjType = ((SJType) super.type()).treeClone();

		if (child != null)
		{
			SJType childType = child.type().treeClone();

			if (this instanceof SJProtocolNode) // Because protocol nodes represent full SJTypes, not just a single node.
			{
				sjType = sjType.append(childType);
			}
			else // This has become an optimisation.
			{
				sjType = sjType.child(childType);
			}
		}

		return sjType;
	}

	public SJTypeNode type(SJType sjType)
	{
		super.type((sjType == null) ? null : sjType.treeClone());

		return this;
	}

	public String toString()
	{
		return treeToString();
	}

	public String treeToString()
	{
		return (child() == null) ? nodeToString() : nodeToString() + SJ_STRING_SEPARATOR + child().treeToString();
	}

	// Can't call type().toString as (session) types may not have been built yet.
	public void prettyPrint(CodeWriter w, PrettyPrinter tr)
	{
		w.write(toString());
  }
}
