/*
 * This file is part of SessionJ.
 *
 * Copyright (C) Raymond Hu, Imperial College London.
 *
 */

package sj.del;

import java.util.LinkedList;
import java.util.List;

import polyglot.ast.*;
import polyglot.types.*;
import polyglot.visit.*;

import sj.SJConstants;
import sj.ast.*;
import sj.ast.sessvars.*;
import sj.types.SJTypeSystem;
import sj.util.SJTypeBuilder;

public class SJSessionTryDel_c extends JL_c implements SJSessionTryDel
{
	public Node typeCheck(TypeChecker tc) throws SemanticException
	{
		SJTypeSystem sjts = (SJTypeSystem) tc.typeSystem();
		SJNodeFactory sjnf = (SJNodeFactory) tc.nodeFactory();

		Context context = tc.context();

		SJTypeBuilder sjtb = new SJTypeBuilder(tc.job(), sjts, sjnf);

		SJSessionTry sjst = (SJSessionTry) node();

		List<Receiver> translated = new LinkedList<Receiver>();

		for (Receiver sock : sjst.sockets())
		{
			sock = (Receiver) sjtb.buildTypes(context, sock);

			if (!sock.type().isSubtype(SJConstants.SJSOCKET_TYPE))
			{
				throw new SemanticException("Session-try is for session sockets: " + sjst);
			}

			SJSocketVariable sjsv = null;

			if (sock instanceof Field)
			{
				throw new SemanticException("Field session sockets not allowed: " + sjst);
			}
			else //if (sock instanceof Local)
			{
				sjsv = sjnf.SJLocalSocket(sock.position(), ((Local) sock).id());
			}

			translated.add((Receiver) sjtb.buildTypes(context, sjsv));
		}

		sjst.sockets(translated);

		return sjst.typeCheck(tc); // Still typecheck node as originally intended.
	}
}
