/*
 * This file is part of SessionJ.
 *
 * Copyright (C) Raymond Hu, Imperial College London.
 *
 */

package sj.visit.contexts;

import java.util.*;

import polyglot.ast.*;
import polyglot.types.*;

import sj.types.typeobjects.sesstypes.SJType;
import sj.types.typeobjects.typeableinstances.*;
import sj.visit.contexts.sesscontexts.*;
import sj.util.*;

public interface SJContext
{
	public SJContextElement currentContext();
	public SJMethodContext currentSJMethod() throws SemanticException;
	public SJContextElement parentContext(); // The context of the 'parent' node (has just been popped).
	public void addProtocol(SJNamedInstance sjni);
	public void addChannel(SJNamedInstance sjni);
	public void addSocket(SJNamedInstance sjni);
	public void addServer(SJNamedInstance sjni);

	public SJNamedInstance findProtocol(String name) throws SemanticException;
	public SJNamedInstance findChannel(String name) throws SemanticException;
	public SJNamedInstance findSocket(String name) throws SemanticException;
	public SJNamedInstance findServer(String name) throws SemanticException;

	public SJMethodInstance findSJMethod(String name, List<SJType> argTypes) throws SemanticException;
	public SJThreadInstance findSJThread(String name) throws SemanticException;

	public SJType lookupRecursionArg(SJLabel lab, int argpos) throws SemanticException; // Hacky: to support mutually recursive methods.

	public SJType protocolType(String name) throws SemanticException; // Static info.
	public SJType channelType(String name) throws SemanticException;
	public SJType socketType(String name) throws SemanticException;
	public SJType serverType(String name) throws SemanticException;

	public Set<String> sessionsInScope() throws SemanticException;

	public boolean socketInScope(String name);
	public boolean sessionOpen(String session); // Includes "completed" (sub)sessions. // Maybe this should be changed to exclude completed sessions (the session type checker would have to check for completion and update the context).
	public boolean sessionInScope(String session) throws SemanticException;

	public SJType activeType(String session) throws SemanticException;
	public SJType implementedType(String session) throws SemanticException;
	public SJType expectedType(String session) throws SemanticException;

	public void activeType(String session, SJType sjType) throws SemanticException;
	public void implementedType(String session, SJType sjType) throws SemanticException;
	public void openSession(String session, SJType sjType) throws SemanticException;
	public void advanceSession(String session, SJType sjType) throws SemanticException;
	public void updateContext(SJSessionContext sce) throws SemanticException;
	public void completeSession(String session) throws SemanticException;
	public void delegateSession(String session) throws SemanticException;

	public void pushClass(ClassDecl cd) throws SemanticException;
	public void pushCode();
	public void pushTry();
	public void pushSJSessionTry(Set<String> sockets) throws SemanticException;
	public void pushIf();
	public void pushIfBranch();
	public void pushWhile();

	public void pushSJOutbranch(String session, SJLabel lab) throws SemanticException;
	public void pushSJInbranch(String session) throws SemanticException;
	public void pushSJInbranchCase(SJLabel lab) throws SemanticException;
	public void pushSJOutwhile(String session) throws SemanticException;
	public void pushSJInwhile(String session) throws SemanticException;
	public void pushSJRecursion(String session, SJLabel lab) throws SemanticException;
	public void pushSJMethodDecl(SJMethodInstance sjmi);
	public void pushSJRecursiveMethodDecl(SJMethodInstance sjrmi);

	public SJContextElement pop();
	public SJContextElement popBranchCaseContext();
}
