/*
 * This file is part of SessionJ.
 *
 * Copyright (C) Raymond Hu, Imperial College London.
 *
 */

package sj.visit.contexts;

import java.util.*;

import sj.types.typeobjects.typeableinstances.*;

public abstract class SJContextElement_c implements SJContextElement
{
	private List<SJNamedInstance> protocols = new ArrayList<SJNamedInstance>();
	private List<SJNamedInstance> channels = new ArrayList<SJNamedInstance>();
	private List<SJNamedInstance> sockets = new ArrayList<SJNamedInstance>();
	private List<SJNamedInstance> servers = new ArrayList<SJNamedInstance>();

	public void addProtocol(SJNamedInstance sjni)
	{
		protocols.add(sjni); // No need to check for name clases due to base typing.
	}

	public void addChannel(SJNamedInstance sjni)
	{
		channels.add(sjni);
	}

	public void addSocket(SJNamedInstance sjni)
	{
		sockets.add(sjni);
	}

	public void addServer(SJNamedInstance sjni)
	{
		servers.add(sjni);
	}

	public boolean hasProtocol(String name)
	{
		for (SJNamedInstance sjni : protocols)
		{
			if (sjni.name().equals(name))
			{
				return true;
			}
		}

		return false;
	}

	public boolean hasChannel(String name)
	{
		for (SJNamedInstance sjni : channels)
		{
			if (sjni.name().equals(name))
			{
				return true;
			}
		}

		return false;
	}

	public boolean hasSocket(String name)
	{
		for (SJNamedInstance sjni : sockets)
		{
			if (sjni.name().equals(name))
			{
				return true;
			}
		}

		return false;
	}

	public boolean hasServer(String name)
	{
		for (SJNamedInstance sjni : servers)
		{
			if (sjni.name().equals(name))
			{
				return true;
			}
		}

		return false;
	}

	public SJNamedInstance getProtocol(String name)
	{
		for (SJNamedInstance sjni : protocols)
		{
			if (sjni.name().equals(name))
			{
				return sjni;
			}
		}

		return null; // Would it be better to throw an exception here?
	}

	public SJNamedInstance getChannel(String name)
	{
		for (SJNamedInstance sjni : channels)
		{
			if (sjni.name().equals(name))
			{
				return sjni;
			}
		}

		return null;
	}

	public SJNamedInstance getSocket(String name)
	{
		for (SJNamedInstance sjni : sockets)
		{
			if (sjni.name().equals(name))
			{
				return sjni;
			}
		}

		return null;
	}

	public SJNamedInstance getServer(String name)
	{
		for (SJNamedInstance sjni : servers)
		{
			if (sjni.name().equals(name))
			{
				return sjni;
			}
		}

		return null;
	}
}
