/*
 * This file is part of SessionJ.
 *
 * Copyright (C) Raymond Hu, Imperial College London.
 *
 */

package sj.visit.contexts.sesscontexts;

import java.util.*;

import sj.types.typeobjects.sesstypes.SJType;
import sj.util.*;
import sj.visit.contexts.*;

public class SJSessionContext_c extends SJCodeContext_c implements SJSessionContext
{
	private SJMap original = new SJMap();
	private SJMap active = new SJMap();
	private SJMap implemented = new SJMap();

	private Set<String> sessionsInScope = new HashSet<String>();

	public SJSessionContext_c()
	{

	}

	public SJSessionContext_c(SJSessionContext sjsc)
	{
		for (String s : sjsc.activeSessions())
		{
			SJType active = sjsc.activeType(s);

			this.originalType(s, (active == null) ? null : active.treeClone());
			this.activeType(s, (active == null) ? null : active.treeClone());
		}
	}

	public Set<String> activeSessions()
	{
		return active.keySet();
	}

	public Set<String> implementedSessions()
	{
		return implemented.keySet();
	}

	public Set<String> sessionsInScope()
	{
		return sessionsInScope;
	}

	public boolean sessionOpen(String session) // Includes "completed" (sub)sessions.
	{
		return original.containsKey(session); // Can use either original or active (but not implemented).
	}

	public SJType originalType(String session)
	{
		SJType sjt = original.get(session);

		return (sjt == null) ? null : sjt.treeClone();
	}

	public SJType activeType(String session)
	{
		SJType sjt = active.get(session);

		return (sjt == null) ? null : sjt.treeClone();
	}

	public SJType implementedType(String session)
	{
		SJType sjt = implemented.get(session);

		return (sjt == null) ? null : sjt.treeClone();
	}

	public boolean sessionInScope(String session)
	{
		return sessionsInScope().contains(session);
	}

	public void openSession(String session, SJType sjType)
	{
		sessionsInScope.add(session); // No check for session-already-open because SJContext uses this operation to overwite information when session-scopes are entered. This check is made in the openSession operation of SJContext instead.
		originalType(session, (sjType == null) ? null : sjType.treeClone());
		activeType(session, (sjType == null) ? null : sjType.treeClone());
		implementedType(session, null);
	}

	private void originalType(String session, SJType sjType)
	{
		original.put(session, (sjType == null) ? null : sjType.treeClone());
	}

	public void activeType(String session, SJType sjType)
	{
		active.put(session, (sjType == null) ? null : sjType.treeClone());
	}

	public void implementedType(String session, SJType sjType)
	{
		implemented.put(session, (sjType == null) ? null : sjType.treeClone());
	}

	public void removeSession(String session)
	{
		original.remove(session);
		active.remove(session);
		implemented.remove(session);
		sessionsInScope().remove(session);
	}
}
