/*
 * This file is part of SessionJ.
 *
 * Copyright (C) Raymond Hu, Imperial College London.
 *
 */

package sj.runtime.net;

import java.io.IOException;
import java.net.*;

import sj.runtime.*;

public final class SJRServerSocket extends SJServerSocket // (Almost) exactly the same as SJFServerSocket. The only difference is that runtime state tracking is mandatory.
{
	public static final SJRServerSocket create(SJProtocol protocol, int port) throws IOException
	{
		return new SJRServerSocket(protocol, port);
	}

	private SJRServerSocket(SJProtocol protocol, int port) throws IOException
	{
		super(protocol, port);
	}

	public final SJRSocket accept() throws SJIOException, SJIncompatibleSessionException
	{
		checkClassLoadingParameters();

		Socket socket = null;

		try
		{
			socket = getServer().accept();
			socket.setTcpNoDelay(SJSocket.TCP_NO_DELAY);
		}
		catch (IOException ioe)
		{
			throw new SJIOException(ioe);
		}

		SJRSocket rsocket = new SJRSocket(getSJType(), getProtocol(), socket);

		rsocket.setRemoteClassLoading(remoteClassLoadingEnabled());
		rsocket.setEagerClassVerification(eagerClassVerificationEnabled());
		rsocket.setEagerClassDownloading(eagerClassDownloadingEnabled());
		rsocket.setCodebase(getCodebase());

		rsocket.acceptIOStreams();

		//SJRuntimeUtils.debugPrintln("Accepted connection on " + getLocalAddress().getPort() + " from: " + socket.getInetAddress().getHostName() + ":" + socket.getPort());

		rsocket.init();
		rsocket.isActive(true);

		if (SJRSocket.RUNTIME_STATE)
		{
			rsocket.state.accept();

			//SJRuntimeUtils.debugPrintln("Current state: " + rsocket.state.currentState());
		}

		return rsocket;
	}
}
