/*
 * This file is part of SessionJ.
 *
 * Copyright (C) Raymond Hu, Imperial College London.
 *
 */

package sj.runtime.net;

import java.io.IOException;
import java.net.*;

import sj.runtime.*;
import sj.runtime.util.*;
import sj.types.typeobjects.sesstypes.SJType;

public abstract class SJServerSocket extends SJRemoteClassHandler
{
	private int port; // Move some of these into the base class?
	private SJProtocol protocol;
	private SJType sjtype;
	private ServerSocket server;

	protected SJServerSocket(SJProtocol protocol, int port) throws IOException // Convert to SJIOException?
	{
		this.port = port;
		this.protocol = protocol;
		this.sjtype = decodeSJType(protocol);

		server = new ServerSocket(port);
	}

	abstract public SJSocket accept() throws SJIOException, SJIncompatibleSessionException;

	public void close() throws IOException // Convert to SJIOException?
	{
		SJRuntimeUtils.closeServerSocket(server);
	}

	public final SJProtocol getProtocol()
	{
		return protocol;
	}

	public final SJType getSJType()
	{
		return sjtype;
	}

	protected final ServerSocket getServer()
	{
		return server;
	}

	public final InetSocketAddress getLocalAddress()
	{
		return new InetSocketAddress(server.getInetAddress(), server.getLocalPort());
	}

	public final boolean isClosed()
	{
		return (getServer() == null) ? true : getServer().isClosed();
	}
}
