//$ bin/sjc -cp tests/classes/ tests/src/rchat/RChatServer.sj -d tests/classes/
//$ bin/sj -cp tests/classes/ rchat.RChatServer 8888

package rchat;

import java.io.*;
import java.util.*;

import sj.runtime.*;
import sj.runtime.net.*;

//import rchat.events.*;

public class RChatServer
{
	private protocol p_recmsgs { begin.?[?(String)]* }
	private protocol p_sendmsgs { begin.?[!<String>]* }	

	private protocol p_events 
	{ 
		rec X[
			?{
				PING: 
					![
						!{
							USER_JOINED: !<int>.!<String>, 
							USER_LEFT: !<int>
						}
					]*
					.#X, 
				BYE: 
			}
		] 
	} 

	private protocol p_control 
	{ 
		begin
		.!<int>.?(String)
		.?(@p_recmsgs)
		.?(@p_sendmsgs)
		.@p_events
	}

	private final EventLog events = new EventLog();
	private final MessageLog log = new MessageLog();	

	private int cid = 0;

	public RChatServer(int port) throws Exception
	{
		SJServerSocket ss_control = null;		
		
		try
		{
			ss_control = SJFServerSocket.create(p_control, port);
					
			for ( ; true; cid++)
			{
				SJSocket s_control = null;
			
				SJSocket s_recmsgs = null;
				SJSocket s_sendmsgs = null;			
			
				try (s_control, s_recmsgs, s_sendmsgs)
				{
					s_control = ss_control.accept();
					
					s_control.send(cid);					
					
					String cname = s_control.receive();
									
					SJServerAddress c_recmsgs = (@p_recmsgs) s_control.receive();
					SJServerAddress c_sendmsgs = (@p_sendmsgs) s_control.receive();
					
					s_recmsgs = SJFSocket.create(c_recmsgs);
					s_sendmsgs = SJFSocket.create(c_sendmsgs);				
					
					// Need a session type parallel construct.
					<s_recmsgs>.spawn(new ReceiveMessagesThread(cid, cname)); 
					<s_sendmsgs>.spawn(new SendMessagesThread(cid));										
					
					<s_control>.spawn(new ControlThread(cid));				
					
					System.out.println("Client connected: " + cname + " (" + cid + ")");					
					
					events.addEvent(new UserJoinedEvent(cid, cname)); // events (shared state) needed for communication between indeterminate producers (each client) and consumers (all clients).
				}
				finally
				{
				
				}
			}
		}
		finally
		{
			if (ss_control != null) ss_control.close();
		}		
	}
	
	private class ControlThread extends SJThread
	{
		private int cid;
		
		private int i = 0;
		
		ControlThread(int cid)
		{
			this.cid = cid;
		}

		// rec X[?{PING: ![!{USER_JOINED: !<int>.!<String>, USER_LEFT: !<int>}]*.#X, BYE: }]
		public void run(@p_events s_control) throws Exception
		{
			s_control.recursion(X)
			{
				s_control.inbranch()
				{
					case PING:
					{									
						RCEvent e = events.getEvent(i);
						
						s_control.outwhile(e != null)
						{
							if (e instanceof UserJoinedEvent)
							{
								s_control.outbranch(USER_JOINED)
								{
									UserJoinedEvent uje = (UserJoinedEvent) e;
								
									s_control.send(uje.getClientId());
									s_control.send(uje.getClientName());
								}
							}
							else 
							{
								if (!(e instanceof UserLeftEvent))
								{
									throw new RuntimeException("Unknown event type: " + e);
								}
							
								s_control.outbranch(USER_LEFT)
								{
									s_control.send(((UserLeftEvent) e).getClientId());
								}							
							}
												
							e = events.getEvent(++i);
						}
						
						s_control.recurse(X);
					}
					case BYE:
					{
						events.addEvent(new UserLeftEvent(cid));
						
						System.out.println("Client (" + cid + ") said: BYE");
					}
				}
			}
		}
	}
	
	private class ReceiveMessagesThread extends SJThread
	{	
		private int cid;
		private String cname;

		ReceiveMessagesThread(int cid, String cname)
		{
			this.cid = cid;
			this.cname = cname;
		}

		public void run(@p_recmsgs s_recmsgs) throws Exception
		{	
			s_recmsgs.request();
		
			s_recmsgs.inwhile()
			{
				String msg = s_recmsgs.receive();				

				System.out.println("(" + cid + ") received: " + msg);				

				log.putmsg(getTimeStamp() + " <" + cname + "> " + msg);				
			}
		}
	}

	private class SendMessagesThread extends SJThread
	{
		private int cid;
		private String cname;
		private int i = 0;

		SendMessagesThread(int cid)
		{
			this.cid = cid;
		}	

		public void run(@p_sendmsgs s_sendmsgs) throws Exception
		{
			s_sendmsgs.request();
		
			s_sendmsgs.inwhile()
			{
				String msg = log.getmsg(i++);

				System.out.println("(" + cid + ") sending: " + msg);

				s_sendmsgs.send(msg);				
			}
		}
	}		
	
	public static void main(String[] args) throws Exception
	{
		new RChatServer(Integer.parseInt(args[0]));
	}

	private String getTimeStamp()
	{
		Calendar c = new GregorianCalendar();

		String year = "" + c.get(Calendar.YEAR);
		
		String month = null;

		switch (c.get(Calendar.MONTH))
		{
			case Calendar.JANUARY: month = "01"; break;
			case Calendar.FEBRUARY: month = "02"; break;
			case Calendar.MARCH: month = "03"; break;
			case Calendar.APRIL: month = "04"; break;
			case Calendar.MAY: month = "05"; break;
			case Calendar.JUNE: month = "06"; break;
			case Calendar.JULY: month = "07"; break;
			case Calendar.AUGUST: month = "08"; break;
			case Calendar.SEPTEMBER: month = "09"; break;
			case Calendar.OCTOBER: month = "10"; break;
			case Calendar.NOVEMBER: month = "11"; break;
			case Calendar.DECEMBER: month = "12"; break;
		}
		
		int d = c.get(Calendar.DAY_OF_MONTH);		
		String day = ((d < 10) ? "0" : "") + d;
		
		int h = c.get(Calendar.HOUR_OF_DAY);
		String hour = ((h < 10) ? "0" : "") + h;
		
		int m = c.get(Calendar.MINUTE);
		String minute = ((m < 10) ? "0" : "") + m;
		
		return "[" + year + month + day + "-" + hour + minute + "]";	
	}

	private class EventLog 
	{
		private LinkedList events = new LinkedList();
	
		public synchronized void addEvent(RCEvent e)
		{
			events.add(e);
		}	

		public synchronized RCEvent getEvent(int i)
		{
			if (i >= events.size())
			{			
				return null;
			}
			else 
			{
				return (RCEvent) events.get(i);				
			}
		}			
	}
}

abstract class RCEvent implements Serializable
{

}

class UserJoinedEvent extends RCEvent
{
	private int cid;
	private String cname;
	
	public UserJoinedEvent(int cid, String cname)
	{
		this.cid = cid;
		this.cname = cname;
	}
	
	public int getClientId()
	{
		return cid;
	}

	public String getClientName()
	{
		return cname;
	}
}

class UserLeftEvent extends RCEvent
{
	private int cid;
	
	public UserLeftEvent(int cid)
	{
		this.cid = cid;	
	}
	
	public int getClientId()
	{
		return cid;
	}
}
